@@ -65,10 +65,10 @@ class Operation(ABC):
     def compute_fields(self, event: Event,
                        below_event: Optional[Event]) -> None:
         if below_event is not None:
-            event.other_inside_on_left = (below_event.other_inside_on_left
-                                          if (event.from_left
-                                              is below_event.from_left)
-                                          else below_event.inside_on_left)
+            event.other_interior_to_left = (below_event.other_interior_to_left
+                                            if (event.from_left
+                                                is below_event.from_left)
+                                            else below_event.interior_to_left)
         event.in_result = self.in_result(event)
 
     def detect_intersection(self, below_event: Event, event: Event) -> bool:
@@ -129,9 +129,9 @@ class Operation(ABC):
 
     def divide_segment(self, event: Event, point: Point) -> None:
         left_event = Event(False, point, event.complement, event.from_left,
-                           event.inside_on_left)
+                           event.interior_to_left)
         right_event = Event(True, point, event, event.from_left,
-                            event.inside_on_left)
+                            event.interior_to_left)
         event.complement.complement, event.complement = left_event, right_event
         self._events_queue.push(left_event)
         self._events_queue.push(right_event)
@@ -184,13 +184,14 @@ class Operation(ABC):
                           from_left: bool) -> None:
         events_queue = self._events_queue
         for start, end in segments:
-            inside_on_left = True
+            interior_to_left = True
             if start > end:
                 start, end = end, start
-                inside_on_left = False
-            start_event = Event(False, start, None, from_left, inside_on_left)
+                interior_to_left = False
+            start_event = Event(False, start, None, from_left,
+                                interior_to_left)
             end_event = Event(True, end, start_event, from_left,
-                              inside_on_left)
+                              interior_to_left)
             start_event.complement = end_event
             events_queue.push(start_event)
             events_queue.push(end_event)
