@@ -7,14 +7,15 @@ from numbers import (Integral,
                      Real)
 from typing import (Any,
                     Iterable,
-                    List,
                     Sequence,
                     Tuple,
                     Type,
                     TypeVar)
 
+from robust.angular import (Orientation,
+                            orientation)
+
 from clipping.hints import (Base,
-                            BoundingBox,
                             Contour,
                             Coordinate,
                             Multipolygon,
@@ -43,25 +44,38 @@ def to_multipolygon_contours(multipolygon: Multipolygon) -> Iterable[Contour]:
         yield from holes
 
 
-def to_bounding_box(points: Iterable[Point]) -> BoundingBox:
-    points = iter(points)
-    x_min, y_min = x_max, y_max = next(points)
-    for x, y in points:
-        x_min, x_max = min(x_min, x), max(x_max, x)
-        y_min, y_max = min(y_min, y), max(y_max, y)
-    return x_min, x_max, y_min, y_max
+def polygon_to_oriented_segments(polygon: Polygon) -> Iterable[Segment]:
+    border, holes = polygon
+    yield from contour_to_oriented_segments(border,
+                                            clockwise=False)
+    for hole in holes:
+        yield from contour_to_oriented_segments(hole,
+                                                clockwise=True)
 
 
-def are_bounding_boxes_disjoint(left: BoundingBox, right: BoundingBox) -> bool:
-    left_x_min, left_x_max, left_y_min, left_y_max = left
-    right_x_min, right_x_max, right_y_min, right_y_max = right
-    return (left_x_min > right_x_max or left_x_max < right_x_min
-            or left_y_min > right_y_max or left_y_max < right_y_min)
+def contour_to_oriented_segments(contour: Contour,
+                                 *,
+                                 clockwise: bool = False) -> Iterable[Segment]:
+    return (((contour[index - 1], contour[index])
+             for index in range(len(contour)))
+            if (to_contour_orientation(contour)
+                is (Orientation.CLOCKWISE
+                    if clockwise
+                    else Orientation.COUNTERCLOCKWISE))
+            else ((contour[index], contour[index - 1])
+                  for index in range(len(contour) - 1, -1, -1)))
 
 
-def to_segments(contour: Contour) -> List[Segment]:
-    return [(contour[index], contour[(index + 1) % len(contour)])
-            for index in range(len(contour))]
+def contour_to_segments(contour: Contour) -> Iterable[Segment]:
+    return ((contour[index - 1], contour[index])
+            for index in range(len(contour)))
+
+
+def to_contour_orientation(contour: Contour) -> Orientation:
+    index = min(range(len(contour)),
+                key=contour.__getitem__)
+    return orientation(contour[index], contour[index - 1],
+                       contour[(index + 1) % len(contour)])
 
 
 def to_mixed_base(multisegment: Multisegment,
